{{
**  GE Color Effects Control Object
**  Version 0.2, 2010-12-02
**  by Robert Quattlebaum <darco@deepdarc.com>
**
**  http://www.deepdarc.com/2010/11/27/hacking-christmas-lights/
**  
**  This object file is public domain. You may use it as you see fit.
MODIFIED BY PAUL MATOS
}}
CON { General constants }
           
    CMD_LEN             = 26
    MAX_COLOR_VALUE     = $F
    MAX_HUE             = (MAX_COLOR_VALUE+1)*6-1

    DEFAULT_INTENSITY   = $CC   ' Default controler never uses larger than this.
    MAX_INTENSITY       = $FF
    
    MAX_BULB            = $3E   ' Theoretical last possible bulb address. Note
                                ' that normally there are only 50 bulbs on a string.
    BROADCAST_BULB      = $3F   ' Intensity set on this bulb affects all bulbs.
    


CON { These constants are needed only for the stand alone test mode. }
    _clkmode    = xtal1 + pll16x
    _xinfreq    = 5_000_000

VAR
    long cog
    long command



PUB start(pin)
{{ Starts the cog for the object using the given pin index for output. }}
    ' Stop the object if it is already running
    stop
    
    ' Set up pin
    'output_pin_mask := (|< pin)|(|< pin+1)|(|< pin+2)

    output_pin_mask := |< pin

    ' Set up # of bulbs         added by PJM
    no_bulbs := 36              'length of string

    ' ***FOR YBOX2 ONLY***
    ' output_pin_mask := (|< 12)|(|< 13)|(|< 14)
    
    ' Set the timing information
    period_10_us := (clkfreq/1_000_000) * 7
    
    ' Start the cog
    cog := cognew(@cog_init, @command) + 1
    
    ' Make sure the cog actually started
    ifnot cog
        abort -1
        
    return cog

PUB stop
{{ Stops the object. You must call start again before you can use this object. }}
    if cog
        cogstop(cog~ - 1)       ' cog variable is cleared by this statement
    command~



PUB flush
{{ Wait for any prior command to finish. }}
    repeat while command AND (cog > 0)
    
PUB send_raw_command(x)
{{ Sends a single command, consisting of the 26 least significant bits of x. }} 
    ' We use the most significant bit
    ' to help us determine when the command
    ' has been sent by the cog. This works
    ' because the command length is only 26
    ' bits, and the most significant bit won't
    ' be used otherwise.
    'x |= constant(|<31)   'commented out by PJM

    ' Wait for any previous command to finish. 
    'flush                  'commented out by PJM

    ' Set the command. The cog will notice this
    ' and send it automatically, clearing
    ' the command variable when it is finished.
    command := x







DAT { Cog Implementation }
            org

cog_init    
            mov      out_pin_mask, output_pin_mask
            mov      out_pin_mask2, output_pin_mask
            mov      out_pin_mask3, output_pin_mask
            mov      out_pin_mask4, output_pin_mask      
            shl      out_pin_mask2, #1
            shl      out_pin_mask3, #2
            shl      out_pin_mask4, #3
            or      dira, out_pin_mask           'first pin, number provided by output_pin_mask
            or      dira, out_pin_mask2          'second pin, offset by 1 from first pin address
            or      dira, out_pin_mask3          'third pin, offset by 2 from first pin address
            or      dira, out_pin_mask4          'fourth pin, offset by 3 from first pin address

loop        wrlong  zero,par                ' Clear out the previous command

:waitforcmd rdlong  array_ready,par wz  ' Wait for next command
    if_z    jmp     #:waitforcmd
            
            'add     array_ready,#168  
            mov     array_ptr,array_ready       'first byte begin address for first pin
            mov     array_ptr2,array_ready       
            add     array_ptr2, #108            'first byte begin address for second pin
            mov     array_ptr3,array_ready       
            add     array_ptr3, #216            'first byte begin address for third pin
            mov     array_ptr4,array_ready       
            add     array_ptr4, #324            'first byte begin address for fourth pin

            mov     bulb_counter,no_bulbs           
            
             
bulb_loop   

            rdbyte  red_byte,array_ptr  'read red byte
            and     red_byte,#$f0     'mask the upper 4 bits 
            add     array_ptr,#1        'move the arrary pointer to get green byte
            rdbyte  green_byte,array_ptr 'read green byte
            and     green_byte,#$f0     'mask the upper 4 bits 
            add     array_ptr,#1        'move the arrary pointer to get blue byte
            rdbyte  blue_byte,array_ptr 'read blue byte
            and     blue_byte,#$f0    'mask the upper 4 bits
            add     array_ptr,#1        ''move the arrary pointer to get red byte
            
            rdbyte  red_byte2,array_ptr2  'read red byte
            and     red_byte2,#$f0     'mask the upper 4 bits 
            add     array_ptr2,#1        'move the arrary pointer to get green byte
            rdbyte  green_byte2,array_ptr2 'read green byte
            and     green_byte2,#$f0     'mask the upper 4 bits 
            add     array_ptr2,#1        'move the arrary pointer to get blue byte
            rdbyte  blue_byte2,array_ptr2 'read blue byte
            and     blue_byte2,#$f0    'mask the upper 4 bits            
            add     array_ptr2,#1        ''move the arrary pointer to get red byte
            
            rdbyte  red_byte3,array_ptr3  'read red byte        
            and     red_byte3,#$f0     'mask the upper 4 bits  
            add     array_ptr3,#1        'move the arrary pointer to get green byte
            rdbyte  green_byte3,array_ptr3 'read green byte
            and     green_byte3,#$f0     'mask the upper 4 bits 
            add     array_ptr3,#1        'move the arrary pointer to get blue byte
            rdbyte  blue_byte3,array_ptr3 'read blue byte
            and     blue_byte3,#$f0    'mask the upper 4 bits            
            add     array_ptr3,#1        ''move the arrary pointer to get red byte

            rdbyte  red_byte4,array_ptr4  'read red byte        
            and     red_byte4,#$f0     'mask the upper 4 bits  
            add     array_ptr4,#1        'move the arrary pointer to get green byte
            rdbyte  green_byte4,array_ptr4 'read green byte
            and     green_byte4,#$f0     'mask the upper 4 bits 
            add     array_ptr4,#1        'move the arrary pointer to get blue byte
            rdbyte  blue_byte4,array_ptr4 'read blue byte
            and     blue_byte4,#$f0    'mask the upper 4 bits           
            add     array_ptr4,#1        ''move the arrary pointer to get red byte

            shl     blue_byte,#4        'shift the blue bits into the correct position            
            shr     red_byte,#4       'shift the green bits into the correct position
            or      current_command,blue_byte   'place blue bits into current command
            or      current_command,red_byte    'place red bits into current command
            or      current_command,green_byte   'place green bits into current command
            or      current_command,bulb_intensity      'place bulb intensisty bits into current command
                
            shl     blue_byte2,#4        'shift the blue bits into the correct position            
            shr     red_byte2,#4       'shift the green bits into the correct position
            or      current_command2,blue_byte2   'place blue bits into current command
            or      current_command2,red_byte2    'place red bits into current command
            or      current_command2,green_byte2   'place green bits into current command
            or      current_command2,bulb_intensity      'place bulb intensisty bits into current command                       

            shl     blue_byte3,#4        'shift the blue bits into the correct position            
            shr     red_byte3,#4       'shift the green bits into the correct position
            or      current_command3,blue_byte3   'place blue bits into current command
            or      current_command3,red_byte3    'place red bits into current command
            or      current_command3,green_byte3   'place green bits into current command
            or      current_command3,bulb_intensity      'place bulb intensisty bits into current command

            shl     blue_byte4,#4        'shift the blue bits into the correct position            
            shr     red_byte4,#4       'shift the green bits into the correct position
            or      current_command4,blue_byte4   'place blue bits into current command
            or      current_command4,red_byte4    'place red bits into current command
            or      current_command4,green_byte4   'place green bits into current command
            or      current_command4,bulb_intensity      'place bulb intensisty bits into current command 

            mov     bulb_address,no_bulbs           'calculate bulb address            
            sub     bulb_address,bulb_counter
            shl     bulb_address,#20    'shift the bulb address bits into the correct position

            or      current_command,bulb_address        'place bulb address bits into current command
            or      current_command2,bulb_address        'place bulb address bits into current command
            or      current_command3,bulb_address        'place bulb address bits into current command
            or      current_command4,bulb_address        'place bulb address bits into current command


            ' Send the command
            
send_cmd    ' Load number of data bits
            mov     current_bit, #CMD_LEN

            ' Rotate the bits so that the next bit is at bit zero.
            rol     current_command, #(32-CMD_LEN)
            rol     current_command2, #(32-CMD_LEN)
            rol     current_command3, #(32-CMD_LEN)
            rol     current_command4, #(32-CMD_LEN)


            ' Set up next_cnt
            mov     next_cnt, period_10_us
            add     next_cnt, cnt

            ' Send start pulse
            or      outa, out_pin_mask   ' Output high
            or      outa, out_pin_mask2   ' Output high
            or      outa, out_pin_mask3   ' Output high
            or      outa, out_pin_mask4   ' Output high
            waitcnt next_cnt, period_10_us  ' Wait 10 uSeconds
            andn    outa, out_pin_mask   ' Output low
            andn    outa, out_pin_mask2   ' Output low
            andn    outa, out_pin_mask3   ' Output low
            andn    outa, out_pin_mask4   ' Output low

output_loop            
            andn    outa, out_pin_mask   ' Output low
            andn    outa, out_pin_mask2   ' Output low
            andn    outa, out_pin_mask3   ' Output low
            andn    outa, out_pin_mask4   ' Output low


            waitcnt next_cnt, period_10_us  ' Wait 10 uSeconds, first 10uS all pins are always low

            rol     current_command, #1 wc                  'check to see if we need to trasition from low to high yet on each pin
    if_c    andn    outa, out_pin_mask   ' Output low
    if_nc   or      outa, out_pin_mask   ' Output high
     
            rol     current_command2, #1 wc
    if_c    andn    outa, out_pin_mask2   ' Output low
    if_nc   or      outa, out_pin_mask2   ' Output high
    
            rol     current_command3, #1 wc
    if_c    andn    outa, out_pin_mask3   ' Output low
    if_nc   or      outa, out_pin_mask3   ' Output high

            rol     current_command4, #1 wc
    if_c    andn    outa, out_pin_mask4   ' Output low
    if_nc   or      outa, out_pin_mask4   ' Output high
    
            waitcnt next_cnt, period_10_us  ' Wait 10 uSeconds

            
            or      outa, out_pin_mask   ' Output high
            or      outa, out_pin_mask2   ' Output high
            or      outa, out_pin_mask3   ' Output high
            or      outa, out_pin_mask4   ' Output high
            
            waitcnt next_cnt, period_10_us  ' Wait 10 uSeconds   'last 10 uS all pins are always high
        

            ' Decrement current_bit ; jump if not Zero
            djnz    current_bit,    #output_loop

            andn    outa, out_pin_mask   ' Output low
            andn    outa, out_pin_mask2   ' Output low
            andn    outa, out_pin_mask3   ' Output low
            andn    outa, out_pin_mask4   ' Output low

            ' Finish up the frame with 30 uSecond quiet period.
            waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds
            waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds
            waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds
            waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds, last 30uS were added to make it easier to see signal on logic analyzer
            waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds
            'waitcnt     next_cnt, period_10_us  ' Wait 10 uSeconds
            
            mov    current_command,zero             'clear current command
            mov    current_command2,zero             'clear current command
            mov    current_command3,zero             'clear current command
            mov    current_command4,zero             'clear current command

 

            djnz    bulb_counter,    #bulb_loop        'repeat for the next bulb

            ' Jump back to the start and wait for next command
            jmp     #loop

'DAT { Send Command Subroutine }


'send_cmd_ret ret

DAT { Constants }
period_10_us    long    0       ' Set at init time
output_pin_mask long    0       ' Set at init time
no_bulbs        long    0       ' Set at init time
zero            long    0
bulb_intensity  long    %00000000_00001100_11000000_00000000 '$CC, shifted into correct position

DAT { Variables }
next_cnt        res     1
current_command res     1
current_command2 res     1
current_command3 res     1
current_command4 res     1

current_bit     res     1
array_ptr       res     1
array_ptr2      res     1
array_ptr3      res     1
array_ptr4      res     1
       
red_byte        res     1
red_byte2       res     1
red_byte3       res     1
red_byte4       res     1
       
green_byte      res     1
green_byte2     res     1
green_byte3     res     1
green_byte4     res     1
      
blue_byte       res     1
blue_byte2      res     1
blue_byte3      res     1
blue_byte4      res     1
      
bulb_counter    res     1      
array_ready     res     1      
bulb_address    res     1
out_pin_mask    res     1
out_pin_mask2   res     1
out_pin_mask3   res     1
out_pin_mask4   res     1   